'''
Extracts selected displacements for all node sets from given CalculiX DAT-file with their coordinates.

Usage:
	displ4plot fname.inp fname.dat node_coor result

Parameters:
	fname.inp - CCX input file name (used for node coordinates)
	fname.dat - CCX output DAT-file name 
	node_coor - node coordinate component index (1 -> X, 2 -> y, 3 -> z)
	result    - type od the displacement:
		1,2,3 - X,Y or Z component of the node displacement, respectively
		0     - resulting displacents (sqrt(x^2 + y^2 + z^2))
		-1,-2,-3 - X,Y or Z component of the node displacement with the opposite sign
		
Output:
	<nset>.out - test file with two columns: coordinates and displacement values for a node set name <nset>.
----- (C) I.Rokach  ---- v 0.9 ---------------------
'''

import math,string,sys

args = sys.argv[1:]

if not args:
    sys.exit(__doc__)
if len(args)!=4:
    sys.exit(__doc__)

finp = args[0]
if finp[-3:].lower() != "inp":
    print("Extension of the first file ("+ finp + ") is not INP\n")
    sys.exit("Bad INP-file name")
try:
    finp=open(finp,'r')
except:
    print("There is no file "+ finp + "\n")
    sys.exit()

fdat = args[1]
if fdat[-3:].lower() != "dat":
    print("Extension of the second file ("+ fdat + ") is not DAT\n")
    sys.exit("Bad DAT-file name")
try:
    fdat=open(fdat,'r')
except:
    print("There is no file "+ fdat + "\n")
    sys.exit()
	
try: 
    node_coord = int(args[2])
except:
    print("Third argument ("+args[2]+") should be an integer\n")

if node_coord not in range(1,4):
    print("Third argument ("+args[2]+") should be an integer from 1 to 3\n")
    sys.exit()


try: 
    result = int(args[3])
except:
    print("Fourth argument ("+args[3]+") should be an integer\n")

if result not in range(-3,4):
    print("Fourth argument ("+args[3]+") should be an integer from -3 to 3\n")
    sys.exit()

## - Reading of the INP file, extraction of the nodes
input_data=finp.readlines()
node_card_found = False

for x in input_data:
    in_line = x.strip()
    if len(in_line)==0:
        continue

    elif in_line.lower()=="*node":
        node_card_found = True
        nodes = []
        continue

    elif in_line.lower().find("*element")>-1 :
        break

    elif node_card_found:
        nodes.append(in_line.replace(',', ''))

## - Reading of the DAT file, append coordinates to node displacement values

node_data_found = False

for x in fdat:
    in_line = x.strip()
    if len(in_line)==0:
        node_data_found = False
        continue
    if not node_data_found:
        position = in_line.find(" set ")
        if position >0:
            out_file_name = in_line[position::].split()[1].lower()
            fout=open(out_file_name+".out",'w+')
            node_data_found = True
            next(fdat,None) # -- Skip next empty line
            continue
    else:
        result_line = in_line.split()
        node_number = result_line[0]
        for node_line in nodes:
            node_temp = node_line.split()
            if node_number == node_temp[0]:
                if result > 0:
                    out_value = float(result_line[result])
                elif result == 0:
                    out_value = math.sqrt(pow(float(result_line[1]),2) + pow(float(result_line[2]),2) + pow(float(result_line[3]),2))
                else:
                    out_value = -float(result_line[result])
                fout.write(node_temp[node_coord]+" "+str(out_value)+"\n")
                break

finp.close()
fdat.close()
fout.close()